use crate::cpu::op::AddressingMode::*;
use crate::cpu::op::{AddressingMode, Instruction};
use crate::cpu::Cpu;
use simplelog::debug;

impl Cpu {
    pub fn disassemble_next_instr(&self) {
        let registers = self.registers;
        let op_code = self.memory_bus.borrow().get_byte(registers.pc) as usize;

        let instr: Instruction = Self::INSTRUCTIONS[op_code];
        let operation = instr.op().to_string();
        let addr_mode = self.disassemble_addr_mode(instr.addr_mode(), registers.pc);

        debug!(
            "{:#04X} - ${:04X} {} {}\tA {:#04X}, X {:#04X}, Y {:#04X}, SP: {:#04X}, F: {:#04X}",
            op_code,
            registers.pc,
            operation,
            addr_mode,
            registers.a,
            registers.x,
            registers.y,
            registers.sp,
            registers.status
        );
    }

    fn disassemble_addr_mode(&self, addr_mode: AddressingMode, pc: u16) -> String {
        let word_peek = self.memory_bus.borrow().get_word(pc);
        let byte_peek = (word_peek & 0xFF) as u8;

        match addr_mode {
            ZP0 => format!("${:04X}", byte_peek),
            ZPX => format!("${:04X},x", byte_peek),
            ZPY => format!("${:04X},x", byte_peek),
            ABS => format!("${:04X}", word_peek),
            ABX => format!("${:04X},x", word_peek),
            ABY => format!("${:04X},y", word_peek),
            IND => format!("(${:04X})", word_peek),
            IDX => format!("(${:04X},x)", word_peek),
            IDY => format!("(${:04X}),y", word_peek),
            REL => format!("#${:04X}", word_peek),
            ACC => String::from('A'),
            IMP => String::from("I"),
            IMM => format!("{:04X}", byte_peek),
        }
    }
}
