use crate::mappers::nrom::NRom;
use crate::rom::Rom;

mod nrom;

pub trait Mapper {
    /// Gets a byte from the program ROM.
    ///
    /// # Arguments
    ///  * addr - The address of the byte to read
    fn get_prg_byte(&self, addr: u16) -> u8;

    /// Gets a byte from the character ROM.
    ///
    /// # Arguments
    ///  * addr - The address of the byte to read
    fn get_chr_byte(&self, addr: u16) -> u8;
}

/// Gets a mapper for a ROM.
///
/// # Arguments
///  * rom - The ROM to get the mapper for
pub fn get_mapper(rom: Rom) -> Box<dyn Mapper> {
    let instance = match rom.mapper {
        0 => NRom::new(rom),
        _ => unreachable!("Unsupported mapper {}", rom.mapper),
    };

    Box::new(instance)
}